﻿using Syncfusion.UI.Xaml.Diagram;
using Syncfusion.UI.Xaml.Diagram.Controls;
using Syncfusion.UI.Xaml.Diagram.Stencil;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using Windows.UI.Xaml.Data;
using CultureInfo = System.String;
using System.ComponentModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Graphics.Display;
using Windows.Graphics.Imaging;
using Windows.Storage.Streams;
using Windows.UI.Core;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;
using System.Diagnostics;
using Windows.Storage;
using Windows.Storage.FileProperties;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409

namespace Stencil
{

    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {

        #region Properties and fields

        private string imageurl;
        public string ImageUrl
        {
            get
            {
                return imageurl;
            }
            set
            {
                if (imageurl != value)
                {
                    if (value != null)
                    {
                        imageurl = value;
                        OnPropertyChanged(("ImageUrl"));
                    }
                }
            }
        }
        private Brush _mFill = new SolidColorBrush(Windows.UI.Colors.Transparent);
        public Brush Fill
        {
            get
            {
                return _mFill;
            }
            set
            {
                if (_mFill != value)
                {
                    _mFill = value;
                    OnPropertyChanged("Fill");
                }
            }
        }

        private Brush _mstroke = new SolidColorBrush(Windows.UI.Colors.Transparent);
        public Brush Stroke
        {
            get
            {
                return _mstroke;
            }
            set
            {
                if (_mstroke != value)
                {
                    _mstroke = value;
                    OnPropertyChanged("Stroke");
                }
            }
        }
        private double _Thickness = 4;
        public double Thickness
        {
            get
            {
                return _Thickness;

            }
            set
            {
                if (value != _Thickness)
                {
                    _Thickness = value;
                    OnPropertyChanged("Thickness");
                }
            }
        }

        Style style = null;

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string name)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(name));
            }
        }

        private StorageFolder _captureFolder = null;

        #endregion

        #region Cstor

        public MainPage()
        {
            Debug.WriteLine(DateTime.Now.ToString("hh.mm.ss.ffffff") + " : MainPage - MainPage()  - start");
            this.InitializeComponent();

            //diagram.SFSelector.Style = this.Resources["CustomSelector"] as Style;

            //PCD : test => load properties lists
            //LoadColors();
            LoadDefinedColors();
            //LoadThicknessSizes();
            style = new Style();
            diagram.PortVisibility = PortVisibility.Visible;

            //PCD : test => block the image properties
            DiagramCollection<NodeViewModel> nodes = (diagram as SfDiagram).Nodes as DiagramCollection<NodeViewModel>;
            if (nodes != null && nodes.Count > 0)
            {
                int cptNodes = 0;
                foreach (var node in nodes)
                {
                    cptNodes++;
                    if (cptNodes == 1)
                    {
                        node.Constraints = ~NodeConstraints.Rotatable & ~NodeConstraints.Menu &
                                            ~NodeConstraints.Resizable & ~NodeConstraints.Draggable &
                                            ~NodeConstraints.OutConnect & ~NodeConstraints.Connectable &
                                            ~NodeConstraints.Selectable;
                        node.ZIndex = 0;
                     
                    }
                }
            }


            //PCD : test => page properties and layout
            /*
            diagram.Constraints = diagram.Constraints & ~GraphConstraints.PanRails;
            diagram.Constraints |= GraphConstraints.Bridging;
            */

            //To Disable ContextMenu
            diagram.Menu = null;
            //Initialize PageSettings and Snapping 
            InitializeDiagram();

            //diagram.PointerReleased += diagramcontrol_PointerReleased;
            //diagram.Tool = Tool.ZoomPan;

            // Syncfusion => disable selection area
            diagram.PageSettings.PageBorderBrush = new SolidColorBrush(Windows.UI.Colors.Transparent);

            //(diagram.Info as IGraphInfo).ItemAdded += MainWindow_ItemAdded;
            (diagram.Info as IGraphInfo).DragEnter += MainWindow_DragEnter;
            (diagram.SelectedItems as SelectorViewModel).Constraints &= ~NodeConstraints.OutConnect & ~NodeConstraints.Connectable
                                                                     & ~NodeConstraints.InConnect;
            //(diagram.SelectedItems as SelectorViewModel).SelectorConstraints &= ~SelectorConstraints.QuickCommands;
            stencil.SelectedFilter = new SymbolFilterProvider() { SymbolFilter = SymbolFilter };

            

            Debug.WriteLine(DateTime.Now.ToString("hh.mm.ss.ffffff") + " : MainPage - MainPage()  - end");
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            Debug.WriteLine(DateTime.Now.ToString("hh.mm.ss.ffffff") + " : MainPage - OnNavigatedTo()  - start");
            base.OnNavigatedTo(e);

            if (Frame.CanGoBack)
                SystemNavigationManager.GetForCurrentView().AppViewBackButtonVisibility = AppViewBackButtonVisibility.Visible;

            Debug.WriteLine(DateTime.Now.ToString("hh.mm.ss.ffffff") + " : MainPage - OnNavigatedTo()  - end");
        }

        #endregion

        #region Private

        private void InitializeDiagram()
        {
            Debug.WriteLine(DateTime.Now.ToString("hh.mm.ss.ffffff") + " : MainPage - InitializeDiagram()  - start");
            diagram.ScrollSettings.ScrollLimit = ScrollLimit.Diagram;
            //diagram.ScrollSettings.ScrollLimit = ScrollLimit.Limited;

            diagram.PageSettings.PageBorderBrush = new SolidColorBrush(Windows.UI.Colors.Transparent);
            diagram.PageSettings.PageBackground = new SolidColorBrush(Windows.UI.Colors.Transparent);
            Debug.WriteLine(DateTime.Now.ToString("hh.mm.ss.ffffff") + " : MainPage - InitializeDiagram()  - end");
        }

        private void MainWindow_ItemAdded(object sender, ItemAddedEventArgs args)
        {
            if (args.ItemSource == ItemSource.Stencil)
            {
                var dropedItem = args.Item as NodeViewModel;

                if (dropedItem != null)
                {
                    dropedItem.ShapeStyle = getstyle();
                }
            }
        }

        private void MainWindow_DragEnter(object sender, ItemDropEventArgs args)
        {
            if (args.ItemSource == Cause.Stencil)
            {
                var dropedItem = args.Source as Node;

                if (dropedItem != null)
                {
                    //dropedItem.ShapeStyle = GetCustomStyle();
                    //dropedItem.ShapeStyle = getstyle();

                    if (dropedItem.Key != null)
                    {
                        if (thick != 0)
                            Thickness = thick;
                        if (dropedItem.Key.ToString().ToLower().Contains("arrow"))
                        {
                            Fill = selectedColor.ColorBrush;
                            Stroke = selectedColor.ColorBrush;
                            dropedItem.ShapeStyle = GetCustomStyle();
                        }
                        else
                        {
                            Stroke = selectedColor.ColorBrush;
                            Fill = null;
                            dropedItem.ShapeStyle = GetCustomStyle();
                        }
                    }
                }
            }
        }

        //Filter event
        private bool SymbolFilter(SymbolFilterProvider sender, object symbol)
        {
            return true;
        }

        #region -> Style 

        private Style GetCustomStyle()
        {
            Style s = new Style();
            s.TargetType = typeof(Windows.UI.Xaml.Shapes.Path);
            if (Fill!=null )
            {
                s.Setters.Add(new Setter(Windows.UI.Xaml.Shapes.Path.FillProperty, Fill));

            }
            if (!double.IsNaN(Thickness) && !double.IsInfinity(Thickness))
            {
                s.Setters.Add(new Setter(Windows.UI.Xaml.Shapes.Path.StrokeThicknessProperty, Thickness));
            }
            if (Stroke != null)
            {
                s.Setters.Add(new Setter(Windows.UI.Xaml.Shapes.Path.StrokeProperty, Stroke));
            }
            s.Setters.Add(new Setter(Windows.UI.Xaml.Shapes.Path.StretchProperty, Stretch.Fill));

            return s;
        }

        private Style getstyle()
        {
            Style s = this.Resources["shapeStyle1"] as Style;
            style.TargetType = typeof(Windows.UI.Xaml.Shapes.Path);
            if (style.Setters.Count() == 0)
            {
                foreach (Setter set in s.Setters)
                {
                    if (set.Property == Windows.UI.Xaml.Shapes.Path.FillProperty)
                    {
                        Fill = (Brush)set.Value;
                    }
                    if (set.Property == Windows.UI.Xaml.Shapes.Path.StrokeProperty)
                    {
                        Stroke = (Brush)set.Value;
                    }
                    if (set.Property == Windows.UI.Xaml.Shapes.Path.StrokeThicknessProperty)
                    {
                        Thickness = (double)set.Value;
                    }

                }
            }

            return style;
        }

        #endregion

        #region -> Color

        public void LoadColors()
        {
            List<Colors> lColors = new List<Colors>();
            Type type = typeof(Windows.UI.Colors);
            PropertyInfo[] Properties = type.GetRuntimeProperties().ToArray<PropertyInfo>();
            foreach (var item in Properties)
            {
                string s = item.Name;
                Windows.UI.Color c = (Windows.UI.Color)item.GetValue(null, null);
                Colors newcolor = new Colors();
                lColors.Add(new Colors()
                {
                    Color = item.Name,
                    ColorBrush = new SolidColorBrush(c)
                });
            }
            //colorList.ItemsSource = lColors;
        }

        public void LoadDefinedColors()
        {
            //List<Colors> lColors = new List<Colors>();
            //Type type = typeof(Windows.UI.Colors);
            //PropertyInfo[] Properties = type.GetRuntimeProperties().ToArray<PropertyInfo>();
            //foreach (var item in Properties)
            //{
            //    string s = item.Name;
            //    List<string> sList = new List<string>(new string[] { "Red", "Black", "Green", "Blue", "Yellow", "Orange"});
            //    if (sList.Contains(s))
            //    {
            //        Windows.UI.Color c = (Windows.UI.Color)item.GetValue(null, null);
            //        Colors newcolor = new Colors();
            //        lColors.Add(new Colors()
            //        {
            //            Color = item.Name,
            //            ColorBrush = new SolidColorBrush(c)
            //        });
            //    }
            //}

            List<Colors> lColors = new List<Colors>();
            List<string> sList = new List<string>(new string[] { "Red", "Black", "Green", "Blue", "Yellow", "Orange" });

            Type type = typeof(Windows.UI.Colors);
            PropertyInfo[] Properties = type.GetRuntimeProperties().ToArray<PropertyInfo>();
            foreach (var curColor in sList)
            {
                foreach (var item in Properties)
                {
                    string s = item.Name;

                    //if (sList.Contains(s))
                    if (curColor == s)
                    {
                        Windows.UI.Color c = (Windows.UI.Color)item.GetValue(null, null);
                        Colors newcolor = new Colors();
                        lColors.Add(new Colors()
                        {
                            Color = item.Name,
                            ColorBrush = new SolidColorBrush(c)
                        });
                        break;
                    }
                }
            }


            colorList2.ItemsSource = lColors;
            color = lColors[0];
            selectedColor = lColors[0];
            this.colorList2.SelectedItem = selectedColor;
        }

        Colors color = null;
        Colors selectedColor = null;

        private void colorList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            color = (Colors)e.AddedItems[0];
            if (diagram.Nodes != null)
            {
                foreach (NodeViewModel node in diagram.Nodes as DiagramCollection<NodeViewModel>)
                {
                    if (node.IsSelected)
                    {
                        if (node.Key != null)
                        {
                            if (thick != 0)
                                Thickness = thick;
                            if (node.Key.ToString().ToLower().Contains("arrow"))
                            {
                                Fill = color.ColorBrush;
                                Stroke = color.ColorBrush;
                                node.ShapeStyle = GetCustomStyle();
                            }
                            else
                            {
                                Stroke = color.ColorBrush;
                                Fill = null;
                                node.ShapeStyle = GetCustomStyle();
                            }
                        }
                    }
                }
            }
            //myGrid.Background = color.ColorBrush;
        }

        #endregion

        #region -> Thickness
        /*
        public void LoadThicknessSizes()
        {
            List<int> lSizes = new List<int>();

            for (int i = 1; i <= 10; i++)
            {
                lSizes.Add(i);
            }
            thicknessSizeList.ItemsSource = lSizes;
        }
        private void thicknessSizeList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var color = (int)e.AddedItems[0];
            if (diagram.Nodes != null)
            {
                foreach (NodeViewModel node in diagram.Nodes as DiagramCollection<NodeViewModel>)
                {
                    if (node.IsSelected)
                    {
                        Thickness = color;
                        node.ShapeStyle = GetCustomStyle();
                    }
                }
            }
        }
        */
        int thick = 0;
        private void thicknessSizeSlider_ValueChanged(object sender, RangeBaseValueChangedEventArgs e)
        {
             thick = (int)e.NewValue;
            if (diagram != null && diagram.Nodes != null)
            {
                foreach (NodeViewModel node in diagram.Nodes as DiagramCollection<NodeViewModel>)
                {
                    if (node.IsSelected)
                    {                    
                        Thickness = thick;
                        if (color != null && node.Key.ToString().ToLower().Contains("arrow"))
                        {
                            Fill = (color as Colors).ColorBrush;
                            Stroke = (color as Colors).ColorBrush;
                        }
                        else
                            Fill = null;
                        node.ShapeStyle = GetCustomStyle();
                    }
                }
            }
        }

        #endregion

        #region -> Save image

        private async void Button_Click(object sender, RoutedEventArgs e)
        {
            Debug.WriteLine(DateTime.Now.ToString("hh.mm.ss.ffffff") + " : MainPage - Button_Click()  - start");

            // Get image
            var result = await GenerateImage(diagram);

            // Debug "Export"
            var export = await GenerateImage(diagram);
            var picturesLibrary = await StorageLibrary.GetLibraryAsync(KnownLibraryId.Pictures);
            // Fall back to the local app storage if the Pictures Library is not available
            _captureFolder = picturesLibrary.SaveFolder ?? ApplicationData.Current.LocalFolder;
            var file = await _captureFolder.CreateFileAsync("SfDiagram.jpg", CreationCollisionOption.GenerateUniqueName);
            await ReencodeAndSavePhotoAsync(export, file, PhotoOrientation.Normal);

            // Navigate
            Frame.Navigate(typeof(ResultPage), result);
            Debug.WriteLine(DateTime.Now.ToString("hh.mm.ss.ffffff") + " : MainPage - Button_Click()  - end");
        }

        /// <summary>
        /// Save as image
        /// (https://www.syncfusion.com/forums/126075/export-uwp-diagram-to-image)
        /// </summary>
        /// <param name="diagram"></param>
        /// <returns></returns>
        protected async Task<IRandomAccessStream> GenerateImage(UIElement diagram)
        {
            Debug.WriteLine(DateTime.Now.ToString("hh.mm.ss.ffffff") + " : MainPage - GenerateImage()  - start");

            // Render to an image at the current system scale and retrieve pixel contents
            var renderTargetBitmap = new RenderTargetBitmap();

            // unselect all nodes
            //ObservableCollection<TraNode> nodes = ((diagram as SfDiagram).Nodes as ObservableCollection<TraNode>);
            //ObservableCollection<Node> nodes = ((diagram as SfDiagram).Nodes as ObservableCollection<Node>);
            /*
            if (nodes != null && nodes.Count > 0)
            {
                foreach (var node in nodes)
                {
                    node.IsSelected = false;
                }
            }
            */
            DiagramCollection<NodeViewModel> nodes = (diagram as SfDiagram).Nodes as DiagramCollection<NodeViewModel>;
            if (nodes != null && nodes.Count > 0)
            {
                foreach (var node in nodes)
                {
                    node.IsSelected = false;
                }
            }


            await renderTargetBitmap.RenderAsync(diagram);
            var pixelBuffer = await renderTargetBitmap.GetPixelsAsync();

            var pixelBytes = new byte[pixelBuffer.Length];
            using (var reader = DataReader.FromBuffer(pixelBuffer))
            {
                reader.ReadBytes(pixelBytes);
            }

            IRandomAccessStream stream = new InMemoryRandomAccessStream();
            //var encoder = await BitmapEncoder.CreateAsync(BitmapEncoder.PngEncoderId, stream);
            var encoder = await BitmapEncoder.CreateAsync(BitmapEncoder.JpegXREncoderId, stream);

            encoder.SetPixelData(
                BitmapPixelFormat.Bgra8,
                BitmapAlphaMode.Straight,
                (uint)renderTargetBitmap.PixelWidth,
                (uint)renderTargetBitmap.PixelHeight,
                DisplayInformation.GetForCurrentView().LogicalDpi,
                DisplayInformation.GetForCurrentView().LogicalDpi,
                pixelBytes);

            await encoder.FlushAsync();

            Debug.WriteLine(DateTime.Now.ToString("hh.mm.ss.ffffff") + " : MainPage - GenerateImage()  - end");
            return stream;
        }

        #endregion

        #endregion

        /// <summary>
        /// Applies the given orientation to a photo stream and saves it as a StorageFile
        /// </summary>
        /// <param name="stream">The photo stream</param>
        /// <param name="file">The StorageFile in which the photo stream will be saved</param>
        /// <param name="photoOrientation">The orientation metadata to apply to the photo</param>
        /// <returns></returns>
        private static async Task ReencodeAndSavePhotoAsync(IRandomAccessStream stream, StorageFile file, PhotoOrientation photoOrientation)
        {
            using (var inputStream = stream)
            {
                var decoder = await BitmapDecoder.CreateAsync(inputStream);

                using (var outputStream = await file.OpenAsync(FileAccessMode.ReadWrite))
                {
                    var encoder = await BitmapEncoder.CreateForTranscodingAsync(outputStream, decoder);

                    var properties = new BitmapPropertySet { { "System.Photo.Orientation", new BitmapTypedValue(photoOrientation, PropertyType.UInt16) } };

                    await encoder.BitmapProperties.SetPropertiesAsync(properties);
                    await encoder.FlushAsync();
                }
            }
        }

        #region Events

        private void colorList2_ItemClick(object sender, ItemClickEventArgs e)
        {
            //Colors selectedColor = sender as Colors;
            //selectedColor = sender as Colors;
        }

        private void colorList2_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            //Colors selectedColor = sender as Colors;
            //selectedColor = sender as Colors;

        }

        private void Grid_RightTapped(object sender, RightTappedRoutedEventArgs e)
        {
            //Colors selectedColor = (sender as Grid).DataContext as Colors;
            selectedColor = (sender as Grid).DataContext as Colors;
            foreach (NodeViewModel node in diagram.Nodes as DiagramCollection<NodeViewModel>)
            {
                if (node.Key != null)
                {
                    if (thick != 0)
                        Thickness = thick;
                    if (node.Key.ToString().ToLower().Contains("arrow"))
                    {
                        Fill = selectedColor.ColorBrush;
                        Stroke = selectedColor.ColorBrush;
                        node.ShapeStyle = GetCustomStyle();
                    }
                    else
                    {
                        Stroke = selectedColor.ColorBrush;
                        Fill = null;
                        node.ShapeStyle = GetCustomStyle();
                    }
                }
            }
            this.colorList2.SelectedItem = selectedColor;
        }

        private void Grid_Tapped(object sender, TappedRoutedEventArgs e)
        {
            //Colors selectedColor = (sender as Grid).DataContext as Colors;
            selectedColor = (sender as Grid).DataContext as Colors;
            foreach (NodeViewModel node in diagram.Nodes as DiagramCollection<NodeViewModel>)
            {  
                if (node.Key != null)
                {
                    if (thick != 0)
                        Thickness = thick;
                    if (node.Key.ToString().ToLower().Contains("arrow"))
                    {
                        Fill = selectedColor.ColorBrush;
                        Stroke = selectedColor.ColorBrush;
                        node.ShapeStyle = GetCustomStyle();
                    }
                    else
                    {
                        Stroke = selectedColor.ColorBrush;
                        Fill = null;
                        node.ShapeStyle = GetCustomStyle();
                    }
                }
            }
            this.colorList2.SelectedItem = selectedColor;
        }

        #endregion
    }

    public class NodeVM : NodeViewModel
    {

        private string _image = "ms-appx:///Assets/Image.jpeg";

        public string CustomImage
        {
            get { return _image; }
            set
            {
                if (_image != value)
                {
                    _image = value;
                }
            }
        }
    }
    public class PortCollection : ObservableCollection<IPort>
    {

    }
    //Collection of Symbols
    public class SymbolCollection : ObservableCollection<object>
    {

    }
    public class Colors
    {
        public string Color
        {
            get;
            set;
        }

        public Brush ColorBrush
        {
            get;
            set;
        }
    }



    /*
    /// <summary>
    /// Initialize Converter NullToVisibilityConverter derived from IValueConverter
    /// </summary>
    public class NulltoVisibilityConverter : IValueConverter
    {
        /// <summary>
        /// Convert from given type to needed type
        /// </summary>
        /// <param name="value"></param>
        /// <param name="targetType"></param>
        /// <param name="parameter"></param>
        /// <param name="language"></param>
        /// <returns>returns visibility</returns>
        public object Convert(object value, Type targetType, object parameter, CultureInfo language)
        {
            if (value is StencilConstraints)
            {
                StencilConstraints sc = (StencilConstraints)value;
                if (sc.Contains(StencilConstraints.Filters))
                {
                    return Visibility.Visible;
                }
            }


            else if (value is SelectorConstraints && parameter != null)
            {
                SelectorConstraints sc = (SelectorConstraints)value;
                if (parameter.ToString() == "QuickCommands")
                {
                    if (sc.Contains(SelectorConstraints.QuickCommands))
                    {
                        return Visibility.Visible;
                    }
                    else
                    {
                        return Visibility.Collapsed;
                    }
                }
                if (parameter.ToString() == "Rotator")
                {
                    if (sc.Contains(SelectorConstraints.Rotator))
                    {
                        return Visibility.Visible;
                    }
                    else
                    {
                        return Visibility.Collapsed;
                    }
                }
                if (parameter.ToString() == "Resize")
                {
                    if (sc.Contains(SelectorConstraints.Resizer))
                    {
                        return Visibility.Visible;
                    }
                    else
                    {
                        return Visibility.Collapsed;
                    }
                }
            }

            return Visibility.Collapsed;
        }
        /// <summary>
        /// its reverse process of Convert 
        /// </summary>
        /// <param name="value"></param>
        /// <param name="targetType"></param>
        /// <param name="parameter"></param>
        /// <param name="language"></param>
        /// <returns>returns given null type value</returns>
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo language)
        {
            throw new NotImplementedException();
        }
    }

    public class PathConverter : IValueConverter
    {
        /// <summary>
        /// Convert from given type to needed type
        /// </summary>
        /// <param name="value"></param>
        /// <param name="targetType"></param>
        /// <param name="parameter"></param>
        /// <param name="language"></param>
        /// <returns>returns string type value</returns>
        public object Convert(object value, Type targetType, object parameter, CultureInfo language)
        {
            if (value is string)
            {
                return (string)value;
            }
            else
            {
                return value;
            }
        }
        /// <summary>
        /// its reverse process of Convert 
        /// </summary>
        /// <param name="value"></param>
        /// <param name="targetType"></param>
        /// <param name="parameter"></param>
        /// <param name="language"></param>
        /// <returns>returns given object type value</returns>
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo language)
        {
            return value;
        }
    }
    public class InvertAngleConverter : IValueConverter
    {
        /// <summary>
        /// Convert from given type to needed type
        /// </summary>
        /// <param name="value"></param>
        /// <param name="targetType"></param>
        /// <param name="parameter"></param>
        /// <param name="language"></param>
        /// <returns>returns angle</returns>
        public object Convert(object value, Type targetType, object parameter, CultureInfo language)
        {
            if (value is CompositeTransform)
            {
                double ang = (double)(value as CompositeTransform).Rotation();
                return 360 - ang;
            }
            return 0;
        }
        /// <summary>
        /// its reverse process of Convert 
        /// </summary>
        /// <param name="value"></param>
        /// <param name="targetType"></param>
        /// <param name="parameter"></param>
        /// <param name="language"></param>
        /// <returns>returns given object type value</returns>
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo language)
        {
            throw new NotImplementedException();
        }
    }
    internal static class EnumExtensions
    {
        public static bool Contains(this StencilConstraints s, StencilConstraints t)
        {
            return (s & t) != 0;
        }
        public static bool Contains(this SelectorConstraints s, SelectorConstraints t)
        {
            return (s & t) != 0;
        }
    }
    internal static class CompositeTransformExtension
    {
        public static CompositeTransform New()
        {
            return new CompositeTransform();
        }

        public static double TranslateX(this CompositeTransform s)
        {
            return s.TranslateX;
        }
        public static void TranslateX(this CompositeTransform s, double value)
        {
            s.TranslateX = value;
        }

        public static double TranslateY(this CompositeTransform s)
        {
            return s.TranslateY;
        }
        public static void TranslateY(this CompositeTransform s, double value)
        {
            s.TranslateY = value;
        }

        public static double ScaleX(this CompositeTransform s)
        {
            return s.ScaleX;
        }
        public static void ScaleX(this CompositeTransform s, double value)
        {
            s.ScaleX = value;
        }

        public static double ScaleY(this CompositeTransform s)
        {
            return s.ScaleY;
        }
        public static void ScaleY(this CompositeTransform s, double value)
        {
            s.ScaleY = value;
        }

        public static double Rotation(this CompositeTransform s)
        {
            return s.Rotation;
        }
        public static void Rotation(this CompositeTransform s, double value)
        {
            s.Rotation = value;
        }
    }
    */
}
